package reseau;

import java.util.StringTokenizer;

/**
 * L'interface Reseau - Moteur.
 * <p>
 * Cette classe contient les mthodes utilises par le moteur
 * pour effectuer des appels au rseau.
 * Elle s'occupe galement de grer le serveur d'alertes.
 * </p>
 * @author      M2SSI SNMP
 * @version     %I%, %G%
 * @since       1.0
 */ 
public class ReseauMoteur {
	private MibModuleLibrary	lib;
	private	TrapServer			tserv;
	private boolean				tservRunning;
	
	public ReseauMoteur() {
		lib = new MibModuleLibrary();
		tservRunning = false;
		try {
			lib.addModule("mibs/SNMPv2-MIB.txt");
		} catch (MibParseErrorException e) {
			System.out.println("Erreur de chargement de SNMPv2-MIB.txt : " + e);
		}
		try {
			lib.addModule("mibs/IF-MIB.txt");
		} catch (MibParseErrorException e) {
			System.out.println("Erreur de chargement de IF-MIB.txt : " + e);
		}
		try {
			lib.addModule("mibs/IP-MIB.txt");
		} catch (MibParseErrorException e) {
			System.out.println("Erreur de chargement de IP-MIB.txt : " + e);
		}
		try {
			lib.addModule("mibs/TCP-MIB.txt");
		} catch (MibParseErrorException e) {
			System.out.println("Erreur de chargement de TCP-MIB.txt : " + e);
		}
		try {
			lib.addModule("mibs/UDP-MIB.txt");
		} catch (MibParseErrorException e) {
			System.out.println("Erreur de chargement de UDP-MIB.txt : " + e);
		}
	}

/**
 * Initialisation du rseau.
 * <p>
 * L'initialisation doit tre faite par le moteur, une fois celui-ci prt
 *  fonctionner.
 * Elle s'occupe surtout de lancer le serveur d'alerte.
 * </p>
 * 
 * @param mr l'interface <code>MoteurReseau</code> du moteur qui initialise.
 * @return true si l'initialisation s'est bien pass, false sinon.
 * @since 1.0
 */ 
	public boolean init(moteur.MoteurReseau mr) {
		try {
			tserv = new TrapServer(mr);
			tserv.start();
			tservRunning = true;
		} catch (java.net.SocketException e) {
			System.out.println("Erreur de chargement du serveur d'alerte, serveur non dmarr");
		}
		return true;
	}
	
 /**
  * Finalise le rseau pour la prparer  se fermer.
  * <p>
  * Cette mthode termine le thread du serveur d'alerte.
  * </p>
  *
  * @since 1.0
  */ 
	public void finalize() {
		if (tservRunning) {
			tserv.stopRunning();
			tserv.interrupt();
		}
	}
	
/**
 * Envoi d'une requete de type GET.
 * 
 * @param ip l'adresse IP de la machine  interroger.
 * @param community la communaut sur laquelle effectuer la requete.
 * @param oid l'OID  interroger (sous forme numrique ou textuelle).
 * @return une String reprsentant le rsultat de la requete.
 * @since 1.0
 */ 
	public String getRequest(String ip, String community, String oid) throws java.io.IOException, SnmpErrorException, MibNodeNotFoundException {
		SnmpSocket	sock = new SnmpSocket(ip);
		MibNode		node = getNode(oid);
		int			isos = 20;
		
		//System.out.println("get de " + node.getName() + " type : " + node.getCustomType().getType());
		
		if (node.getCustomType().getType() == CustomType.OCTETSTRING) {
			//System.out.println("get de type OctetString");
			isos = 3;
		}
		
		TypeObject	t = new TypeObject(node.toIntString());
		PDU			pdu = new PDU(Type.GET);
		pdu.addVar(t, new TypeNull());
		
		Message		m = new Message(1, community, pdu);
		sock.send(m);
		
		Message		ret		= sock.receive(isos);
		PDU			pdu_ret	= (PDU)ret.get(2);
		
		if (pdu_ret.getErrorStatus() > 0) {
			throw new SnmpErrorException(pdu_ret.getErrorStatus());
		}
		
		TypeSequence	var		= pdu_ret.getVar(0);
		Type			value	= var.get(1);
		
		return value.toString();
	}
	
/**
 * Envoi d'une requete de type SET.
 * 
 * @param ip l'adresse IP de la machine  interroger.
 * @param community la communaut sur laquelle effectuer la requete.
 * @param oid l'OID  interroger (sous forme numrique ou textuelle).
 * @param value une String contenant la valeur  modifier.
 * @since 1.0
 */ 
	public void setRequest(String ip, String community, String oid, String value) throws java.io.IOException, SnmpErrorException, MibNodeNotFoundException {
		SnmpSocket	sock = new SnmpSocket(ip);
		MibNode		node = getNode(oid);
		Type		val  = node.getCustomType().getUsableType(value);
		int			isos = 20;

		if (node.getCustomType().getType() == CustomType.OCTETSTRING)
			isos = 3;

		TypeObject	t = new TypeObject(node.toIntString());
		PDU			pdu = new PDU(Type.SET);
		pdu.addVar(t, val);
		
		Message		m = new Message(1, community, pdu);
		sock.send(m);
		
		Message		ret		= sock.receive(isos);
		PDU			pdu_ret	= (PDU)ret.get(2);
		
		if (pdu_ret.getErrorStatus() > 0) {
			throw new SnmpErrorException(pdu_ret.getErrorStatus());
		}
		
		TypeSequence	var		= pdu_ret.getVar(0);
		//Type			value	= var.get(1);
	}

/**
 * Teste si une machine rpond aux requetes SNMP.
 * 
 * @param ip l'adresse IP de la machine  interroger.
 * @param community la communaut de la machine.
 * @return true si la machine a rpondu, false s'il y a eu une exception quelconque.
 * @since 1.0
 */ 
	public boolean isOnline(String ip, String community) {
		//System.out.println("isOnline : " + ip);
		try {
			SnmpSocket	sock = new SnmpSocket(ip);

			TypeObject	t = new TypeObject(".1.3.6.1");
			PDU			pdu = new PDU(Type.GET);
			pdu.addVar(t, new TypeNull());
		
			Message		m = new Message(1, community, pdu);
			sock.send(m);
			Message		ret		= sock.receive(50);
		} catch (Exception e) {
			//System.out.println("retourne folseuh : " + e);
			return false;
		}
		//System.out.println("retourne trou");
		return true;
	}

/**
 * Recupre un noeud de l'arbre.
 * 
 * @param oid l'OID complet du noeud (sous forme numrique ou textuelle.
 * @return le noeud.
 * @since 1.0
 */ 
	public MibNode getNode (String oid) throws MibNodeNotFoundException {
		StringTokenizer	tok = new StringTokenizer(oid, ".");
		MibNode			n;
		
		n = lib.getRootTree().getRoot();
		
		String s = tok.nextToken();
		if (s.compareTo("1") != 0 && s.compareTo("iso") != 0) {
			throw new MibNodeNotFoundException();
		}
		
		while (tok.hasMoreTokens()) {
			n = n.getChild(tok.nextToken());
		}
		return n;
	}

/**
 * Retourne la racine de l'arbre de la MIB.
 * 
 * @return le noeud racine.
 * @since 1.0
 */ 
	public MibNode getRoot() {
		return lib.getRootTree().getRoot();
	}
	
	/*
	public void addModule (String path) throws MibParseErrorException {
		lib.addModule(path);
	}*/
	
	/*
	public String getNodeName (String oid) throws MibNodeNotFoundException {
		return getNode(oid).getName();
	}
	
	public String getNodeFullName (String oid) throws MibNodeNotFoundException {
		return getNode(oid).toString();
	}
	*/
}
